/*
 * Sunpath.cpp
 *
 *  Created on: 25.11.2013
 *      Author: luoc
 */

#include "Sunpath.h"

Sunpath::Sunpath(float longitude, float latitude) 
{
  this->longitude = longitude;
  this->latitude = latitude;
}

// private functions

boolean Sunpath::leapYear(time_t t)
{
  int y = year(t);
  if ( y % 4 == 0 && y % 100 != 0 || y % 400 == 0 ) 
  {
    return true;
  }
  else
  {
    return false;
  }
}

// public functions

double Sunpath::getAzimuth(time_t currentTimeUTC)
{
  int minutesOfDay = hour(getTimeSolar(currentTimeUTC)) * 60 + minute(getTimeSolar(currentTimeUTC));
  
  if(minutesOfDay < 0) // make shure that the angle is between 0° and 360°
  {
    minutesOfDay += 1440;
  }
  
  if(minutesOfDay >= 1440) // make shure that the angle is between 0° and 360°
  {
    minutesOfDay -= 1440;
  }
  
  return minutesOfDay * 0.25;
}

double Sunpath::getElevationAngle(time_t currentTimeUTC)
{
  time_t currentTimeSolar = getTimeSolar(currentTimeUTC);
  double h; // hours since 12:00
  int d; // days since 21. June
  
  switch(month(currentTimeSolar))
  {
    case 1: 
    d = 193 + day(currentTimeSolar); 
    break;
    
    case 2: 
    d = 224 + day(currentTimeSolar); 
    break;
    
    case 3: 
    d = 252 + day(currentTimeSolar);
    if(leapYear(currentTimeSolar)){ d++; }
    break;
    
    case 4: 
    d = 283 + day(currentTimeSolar); 
    if(leapYear(currentTimeSolar)){ d++; }
    break;
    
    case 5: 
    d = 313 + day(currentTimeSolar);
    if(leapYear(currentTimeSolar)){ d++; } 
    break;
    
    case 6: 
    if(day(currentTimeSolar) < 21)
    {
      d = 344 + day(currentTimeSolar);
      if(leapYear(currentTimeSolar)){ d++; }
    }else
    {
      d = day(currentTimeSolar) - 21;
    } 
    break;
    
    case 7: 
    d = 9 + day(currentTimeSolar); 
    break;
    
    case 8: 
    d = 40 + day(currentTimeSolar); 
    break;
    
    case 9: 
    d = 71 + day(currentTimeSolar); 
    break;
    
    case 10: 
    d = 101 + day(currentTimeSolar); 
    break;
    
    case 11: 
    d = 132 + day(currentTimeSolar); 
    break;
    
    case 12: 
    d = 162 + day(currentTimeSolar); 
    break;
    
    default: d = 0; break;
  }
  
  if(isAM(currentTimeSolar))
  {
    h = hour(currentTimeSolar) + 12;
    if(month(currentTimeSolar) == 6 && day(currentTimeSolar) == 21)
    {
      if(leapYear(currentTimeSolar))
      {
        d = 365;
      }else
      {
        d = 364;
      }
    }else
    {
      d--;
    }
  }else
  {
    h = hour(currentTimeSolar) - 12;
  }
  
   h += double(minute(currentTimeSolar)) / 60.0;

  // I think it's not necessary to explain :P
  return asin(sin(AXIAL_TILT*(PI/180.0))*sin(latitude*(PI/180.0))*cos((2*PI*double(d))/(366))+cos(latitude*(PI/180.0))*sqrt(1.0-square(sin(AXIAL_TILT*(PI/180.0))*cos(sqrt((2*PI*double(d))/(366)))))*cos((2*PI*h)/(24))) * (180/PI); 
}

time_t Sunpath::getTimeSolar(time_t currentTime)
{
	return long(currentTime + longitude * 4 * 60);
}

Sunpath::~Sunpath() {
	// TODO Auto-generated destructor stub
}


